using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Resources;
using System.Text;
using System.Windows.Forms;

using Smdn.Formats.Thbgm;
using Smdn.IO;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public static class ThbgmExtractor {
    private static StringBuilder log = new StringBuilder(1024 * 1024);

    public static StringBuilder Log {
      get { return log; }
    }

    private static ResourceManager resourceManager = new ResourceManager("Smdn.Applications.ThbgmExtractor.Texts", typeof(ThbgmExtractor).Assembly);
    private static CultureInfo resourceCulture = null;

    public static System.Globalization.CultureInfo Culture {
      get { return resourceCulture; }
      set { resourceCulture = value; }
    }

    private static /*readonly*/ ReadOnlyCollection<ProductInfo> products;

    public static ReadOnlyCollection<ProductInfo> Products {
      get { return products; }
    }

    private static /*readonly*/ ReadOnlyCollection<EncoderConfig> encoderConfigs;

    public static ReadOnlyCollection<EncoderConfig> EncoderConfigs {
      get { return encoderConfigs; }
    }

    public static string VersionString {
      get { return string.Format("{0} {1}", Application.ProductName, Application.ProductVersion); }
    }

    [STAThread]
    public static void Main(string[] args)
    {
#if DEBUG
      resourceCulture = System.Globalization.CultureInfo.GetCultureInfo("en");
      Texts.Culture = resourceCulture;
      System.Threading.Thread.CurrentThread.CurrentUICulture = resourceCulture;
#endif

      var logWriter = TextWriter.Synchronized(new StringWriter(log));

#if DEBUG
      Console.SetError(logWriter);

      Console.WriteLine("current directory: {0}", Environment.CurrentDirectory);
      Console.WriteLine("executable directory: {0}", Path.GetDirectoryName(Application.ExecutablePath));
#else
      Console.SetOut(logWriter);
      Console.SetError(logWriter);
#endif

      var prds = new List<ProductInfo>();

      prds.AddRange(ProductInfo.EmbeddedProducts);

#if DEBUG
      var searchDirectory = Path.GetDirectoryName(Application.ExecutablePath);
#else
      var searchDirectory = Path.GetDirectoryName(Application.ExecutablePath);
#endif

      prds.AddRange(LoadProductInfo(searchDirectory));

      products = prds.AsReadOnly();

      Config.Load(searchDirectory, products);

      LoadEncoderConfig(searchDirectory);

      Application.EnableVisualStyles();
      Application.DoEvents();

      using (var dialog = new MainDialog()) {
        if (0 < args.Length) {
          if (PathUtils.AreExtensionEqual(args[0], ExtractionConfig.FileExtension)) {
            dialog.LoadExtractionConfig(args[0], true);
          }
          else {
            var product = ProductInfo.FindMatchedProduct(args[0], products);

            if (product != null) {
              var config = new ExtractionConfig();

              config.Product = product;
              config.BgmSourcePath = product.InstalledBgmSourcePath;
              config.EncoderConfig = EncoderConfig.Empty;
              config.Tracks = product.Tracks.Select(delegate(TrackInfo trackInfo) {
                return new Track(trackInfo);
              }).ToList().AsReadOnly();

              dialog.ExtractOnShown(config);
            }
          }
        }

        Application.Run(dialog);
      }

      Config.Save(products);
    }

    private static List<ProductInfo> LoadProductInfo(string searchDirectory)
    {
      var ret = new List<ProductInfo>();

      try {
        foreach (var file in ProductInfo.SearchProductInfoFiles(searchDirectory, SearchOption.TopDirectoryOnly)) {
          try {
            var product = ProductInfo.LoadFrom(file);

            Console.WriteLine("loaded: {0} ({1})", file, product.Title);

            ret.Add(product);
          }
          catch (Exception ex) {
            Console.Error.WriteLine("file: {0}{1}{2}", file, Environment.NewLine, ex);
          }
        }
      }
      catch (Exception ex) {
        Console.Error.WriteLine(ex);
      }

      return ret;
    }

    private static void LoadEncoderConfig(string searchDirectory)
    {
      try {
        encoderConfigs = EncoderConfig.LoadFrom(Path.Combine(searchDirectory, EncoderConfig.ConfigFileName));
      }
      catch (Exception ex) {
        encoderConfigs = Array.AsReadOnly(new[] {EncoderConfig.Empty});
        Console.Error.WriteLine(ex);
      }
    }

    public static string GetText(string name, params object[] args)
    {
      return string.Format(resourceManager.GetString(name, resourceCulture) ?? string.Empty, args);
    }
  }
}
